"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const DataLoader = require('dataloader');
class MongoEntity {
    constructor(connector, collectionName, cache) {
        this.connector = connector;
        this._collectionName = collectionName;
        if (cache) {
            this._dataLoaderOptions = {
                cacheMap: cache
            };
        }
        else if (MongoEntity.DataLoaderOptions) {
            this._dataLoaderOptions = MongoEntity.DataLoaderOptions;
        }
    }
    get collection() {
        if (!this._collection) {
            this._collection = this.connector.collection(this._collectionName);
        }
        return this._collection;
    }
    clearUpdateCaches(selector) {
        if (this._singleLoader) {
            if (selector._id) {
                this._singleLoader.clear(selector._id);
            }
            else {
                this._singleLoader.clearAll();
            }
        }
    }
    clearInsertCaches(selector) {
        if (this._multiLoader) {
            this._multiLoader.clearAll();
        }
    }
    assignFilter(object, selector, result, include) {
        return (k) => {
            if (selector[k] != include) {
                throw new Error('You cannot combine include and exclude!');
            }
            if (include) {
                if (object[k] !== undefined) {
                    result[k] = object[k];
                }
            }
            else {
                delete (result[k]);
            }
        };
    }
    filter(object, selector) {
        let keys = Object.keys(selector);
        if (keys.length == 0) {
            throw new Error('You need to specify the selector!');
        }
        let include = selector[keys[0]];
        let result = include ? {} : Object.assign({}, object);
        let selectorFunction = this.assignFilter(object, selector, result, include);
        keys.forEach(selectorFunction);
        return result;
    }
    find(selector, fields, skip, limit, timeout) {
        return this.collection.find(selector, fields, skip, limit, timeout);
    }
    findOne(selector, options) {
        return this.collection.findOne(selector, options);
    }
    findOneCached(loader, key, selector) {
        return __awaiter(this, void 0, void 0, function* () {
            if (selector) {
                const result = yield loader.load(key);
                return this.filter(result, selector);
            }
            else {
                console.log(loader);
                return loader.load(key);
            }
        });
    }
    findOneCachedById(id, selector) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._singleLoader) {
                this._singleLoader = new DataLoader((keys) => {
                    return Promise.all(keys.map((loadId) => {
                        return this.collection.findOne({ _id: loadId });
                    }));
                }, this._dataLoaderOptions);
            }
            return this.findOneCached(this._singleLoader, id, selector);
        });
    }
    findManyCached(loader, key, selector) {
        return __awaiter(this, void 0, void 0, function* () {
            if (selector) {
                const result = yield loader.load(key);
                return result.map(r => this.filter(r, selector));
            }
            else {
                return loader.load(key);
            }
        });
    }
    findAllCached(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._multiLoader) {
                this._multiLoader = new DataLoader((param) => {
                    return Promise.all([this.collection.find().toArray()]);
                }, this._dataLoaderOptions);
            }
            return this.findManyCached(this._multiLoader, 'ALL', selector);
        });
    }
    insert(document) {
        this.clearInsertCaches(document);
        return this.collection.insertOne(document);
    }
    insertMany(document) {
        this.clearInsertCaches(document);
        return this.collection.insertMany(document);
    }
    delete(selector, many = false) {
        this.clearInsertCaches(selector);
        if (many) {
            return this.collection.deleteMany(selector);
        }
        return this.collection.deleteOne(selector);
    }
    update(selector, update, options) {
        this.clearUpdateCaches(selector);
        return this.collection.updateOne(selector, update, options);
    }
    createLoader(selectorFunction, options) {
        return new DataLoader((keys) => {
            return Promise.all(keys.map(selectorFunction));
        }, options);
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = MongoEntity;
