"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const mongodb_1 = require('mongodb');
const mongo_entity_1 = require('./mongo_entity');
let db = null;
let name = '';
let host = '127.0.0.1';
let port = '27017';
let disposeTimeout = 200;
let timeoutId = null;
function config(mongoHost = host, mongoPort = port, mongoDisposeTimeout = disposeTimeout) {
    host = mongoHost;
    port = mongoPort;
    disposeTimeout = mongoDisposeTimeout;
}
exports.config = config;
function getDb() {
    return __awaiter(this, void 0, void 0, function* () {
        if (db) {
            return db;
        }
        name = "tmp" + Math.floor(Math.random() * 10000);
        console.log('New connection to ' + name);
        db = yield mongodb_1.MongoClient.connect(`mongodb://${host}:${port}/${name}`);
        global.db = db;
        return db;
    });
}
exports.getDb = getDb;
function withEntity(test, options) {
    return __awaiter(this, void 0, void 0, function* () {
        if (timeoutId != null) {
            clearTimeout(timeoutId);
        }
        const connector = yield getConnector();
        let entities = [];
        if (options && options.entities) {
            for (let i = 0; i < options.entities.length; i++) {
                const option = options.entities[i];
                const name = option.name;
                const type = option.type ? option.type : mongo_entity_1.default;
                let entity = new type(connector, name ? name : `test_${i}`);
                entities.push(entity);
                if (options.entities[i].data) {
                    for (let data of options.entities[i].data) {
                        yield entity.insert(data);
                    }
                }
            }
        }
        else {
            entities.push(new mongo_entity_1.default(connector, 'test'));
        }
        try {
            yield test(...entities);
        }
        catch (ex) {
            throw ex;
        }
        finally {
            yield entities.forEach(e => e.delete({}, true));
            disposeDb();
        }
    });
}
exports.withEntity = withEntity;
function getConnector() {
    return __awaiter(this, void 0, void 0, function* () {
        let myDb = yield getDb();
        return {
            collection(name) {
                return db.collection(name);
            }
        };
    });
}
exports.getConnector = getConnector;
function disposeDb() {
    timeoutId = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
        let myDb = db;
        db = null;
        if (myDb) {
            console.log('tearing down db');
            yield myDb.dropDatabase();
            yield myDb.close();
        }
    }), disposeTimeout);
}
exports.disposeDb = disposeDb;
